@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Activity
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- It models an activity that can be utilised. The activity is similar to a 'Server'
 -- but destined for simulation within 'Net' computation.
@@ -155,7 +155,7 @@ newPreemptibleStateActivity :: Bool
                                -- old state and input
                                -> s
                                -- ^ the initial state
-                              -> Simulation (Activity s a b)
+                               -> Simulation (Activity s a b)
 newPreemptibleStateActivity preemptible provide state =
   do r0 <- liftIO $ newIORef state
      r1 <- liftIO $ newIORef 0
@@ -554,7 +554,7 @@ activitySummary act indent =
        showString "preemption factor (from 0 to 1) = " . shows xf3 .
        showString "\n" .
        showString tab .
-       showString "utilisation time (locked while awaiting the input):\n\n" .
+       showString "utilisation time:\n\n" .
        samplingStatsSummary xs1 (2 + indent) .
        showString "\n\n" .
        showString tab .
