@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Cont
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- This is an internal implementation module that should never be used directly.
 --
@@ -790,7 +790,15 @@ reenterCont c a =
      if not f
        then invokeEvent p $
             enqueueEvent (pointTime p) $
-            resumeCont c a
+            Event $ \p ->
+            do f <- invokeEvent p $
+                    contPreemptionBegun $
+                    contId $ contAux c
+               if not f
+                 then invokeEvent p $
+                      resumeCont c a
+                 else invokeEvent p $
+                      sleepCont c a
        else invokeEvent p $
             sleepCont c a
 
@@ -820,7 +828,7 @@ sleepCont c a =
                        ContPreemptionEnding ->
                          invokeEvent p $
                          enqueueEvent (pointTime p) $
-                         resumeCont c a
+                         reenterCont c a
                        ContPreemptionBeginning ->
                          error "The computation was already preempted: sleepCont."
      writeIORef rh (Just h)
