@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Process
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- This is an internal implementation module that should never be used directly.
 --
@@ -88,6 +88,8 @@ module Simulation.Aivika.Internal.Process
         memoProcess,
         -- * Never Ending Process
         neverProcess,
+        -- * Retrying Computation
+        retryProcess,
         -- * Debugging
         traceProcess) where
 
@@ -673,10 +675,9 @@ timeoutProcessUsingId timeout pid p =
   do s <- liftSimulation newSignalSource
      timeoutPid <- liftSimulation newProcessId
      spawnProcessUsingIdWith CancelChildAfterParent timeoutPid $
-       finallyProcess
-       (holdProcess timeout)
-       (liftEvent $
-        cancelProcessWithId pid)
+       do holdProcess timeout
+          liftEvent $
+            cancelProcessWithId pid
      spawnProcessUsingIdWith CancelChildAfterParent pid $
        do r <- liftIO $ newIORef Nothing
           finallyProcess
@@ -686,7 +687,8 @@ timeoutProcessUsingId timeout pid p =
              (\e ->
                liftIO $ writeIORef r $ Just (Left e)))
             (liftEvent $
-             do x <- liftIO $ readIORef r
+             do cancelProcessWithId timeoutPid
+                x <- liftIO $ readIORef r
                 triggerSignal s x)
      x <- processAwait $ publishSignal s
      case x of
@@ -717,6 +719,11 @@ neverProcess =
   in handleSignal_ signal $ \_ ->
      resumeCont c $ error "It must never be computed: neverProcess"
 
+-- | Retry the current computation as possible, using the specified argument
+-- as a 'SimulationRetry' exception message in case of failure.
+retryProcess :: String -> Process a
+retryProcess = liftEvent . retryEvent
+
 -- | Show the debug message with the current simulation time.
 traceProcess :: String -> Process a -> Process a
 traceProcess message m =
