@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Processor
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The processor of simulation data.
 --
@@ -44,23 +44,29 @@ module Simulation.Aivika.Processor
         joinProcessor,
         -- * Failover
         failoverProcessor,
-        -- * Integrating with Signals
-        signalProcessor,
-        processorSignaling,
+        -- * Integrating with Signals and Channels
+        channelProcessor,
+        processorChannel,
+        queuedChannelProcessor,
+        queuedProcessorChannel,
         -- * Debugging
         traceProcessor) where
 
 import qualified Control.Category as C
 import Control.Arrow
 
+import Data.Monoid
+
 import Simulation.Aivika.Simulation
 import Simulation.Aivika.Dynamics
 import Simulation.Aivika.Event
+import Simulation.Aivika.Composite
 import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
 import Simulation.Aivika.Stream
 import Simulation.Aivika.QueueStrategy
 import Simulation.Aivika.Signal
+import Simulation.Aivika.Channel
 import Simulation.Aivika.Internal.Arrival
 
 -- | Represents a processor of simulation data.
@@ -90,19 +96,22 @@ instance Arrow Processor where
     Processor $ \xys ->
     Cons $
     do (xs, ys) <- liftSimulation $ unzipStream xys
-       runStream $ zipStreamSeq (f xs) ys
+       runStream $ zipStreamParallel (f xs) ys
 
   second (Processor f) =
     Processor $ \xys ->
     Cons $
     do (xs, ys) <- liftSimulation $ unzipStream xys
-       runStream $ zipStreamSeq xs (f ys)
+       runStream $ zipStreamParallel xs (f ys)
 
   Processor f *** Processor g =
     Processor $ \xys ->
     Cons $
     do (xs, ys) <- liftSimulation $ unzipStream xys
-       runStream $ zipStreamSeq (f xs) (g ys)
+       runStream $ zipStreamParallel (f xs) (g ys)
+
+  Processor f &&& Processor g =
+    Processor $ \xs -> zipStreamParallel (f xs) (g xs)
 
 instance ArrowChoice Processor where
 
@@ -141,12 +150,7 @@ arrProcessor = Processor . mapStreamM
 
 -- | Accumulator that outputs a value determined by the supplied function.
 accumProcessor :: (acc -> a -> Process (acc, b)) -> acc -> Processor a b
-accumProcessor f acc =
-  Processor $ \xs -> Cons $ loop xs acc where
-    loop xs acc =
-      do (a, xs') <- runStream xs
-         (acc', b) <- f acc a
-         return (b, Cons $ loop xs' acc') 
+accumProcessor f acc = Processor $ accumStream f acc
 
 -- | Involve the computation with side effect when processing a stream of data.
 withinProcessor :: Process () -> Processor a a
@@ -426,42 +430,92 @@ queueProcessorLoopParallel enqueue dequeue =
 prefetchProcessor :: Processor a a
 prefetchProcessor = Processor prefetchStream
 
--- | Convert the specified signal transform to a processor.
+-- | Convert the specified signal transform, i.e. the channel, to a processor.
 --
 -- The processor may return data with delay as the values are requested by demand.
 -- Consider using the 'arrivalSignal' function to provide with the information
 -- about the time points at which the signal was actually triggered.
 --
 -- The point is that the 'Stream' used in the 'Processor' is requested outside, 
--- while the 'Signal' is triggered inside. They are different by nature. 
+-- while the 'Signal' used in the 'Channel' is triggered inside. They are different by nature. 
 -- The former is passive, while the latter is active.
 --
--- Cancel the processor's process to unsubscribe from the signals provided.
-signalProcessor :: (Signal a -> Signal b) -> Processor a b
-signalProcessor f =
+-- The resulting processor may be a root of space leak as it uses an internal queue to store
+-- the values received from the input signal. Consider using 'queuedChannelProcessor' that
+-- allows specifying the bounded queue in case of need.
+channelProcessor :: Channel a b -> Processor a b
+channelProcessor f =
   Processor $ \xs ->
   Cons $
-  do sa <- streamSignal xs
-     sb <- signalStream (f sa)
-     runStream sb
+  do let composite =
+           do sa <- streamSignal xs
+              sb <- runChannel f sa
+              signalStream sb
+     (ys, h) <- liftEvent $
+                runComposite composite mempty
+     whenCancellingProcess $
+       disposeEvent h
+     runStream ys
 
--- | Convert the specified processor to a signal transform. 
+-- | Convert the specified processor to a signal transform, i.e. the channel. 
 --
 -- The processor may return data with delay as the values are requested by demand.
 -- Consider using the 'arrivalSignal' function to provide with the information
 -- about the time points at which the signal was actually triggered.
 --
 -- The point is that the 'Stream' used in the 'Processor' is requested outside, 
--- while the 'Signal' is triggered inside. They are different by nature.
+-- while the 'Signal' used in the 'Channel' is triggered inside. They are different by nature.
 -- The former is passive, while the latter is active.
 --
--- Cancel the returned process to unsubscribe from the signal specified.
-processorSignaling :: Processor a b -> Signal a -> Process (Signal b)
-processorSignaling (Processor f) sa =
+-- The resulting channel may be a root of space leak as it uses an internal queue to store
+-- the values received from the input stream. Consider using 'queuedProcessorChannel' that
+-- allows specifying the bounded queue in case of need.
+processorChannel :: Processor a b -> Channel a b
+processorChannel (Processor f) =
+  Channel $ \sa ->
   do xs <- signalStream sa
      let ys = f xs
      streamSignal ys
 
+-- | Like 'channelProcessor' but allows specifying an arbitrary queue for storing the signal values,
+-- for example, the bounded queue.
+queuedChannelProcessor :: (b -> Event ())
+                          -- ^ enqueue
+                          -> Process b
+                          -- ^ dequeue
+                          -> Channel a b
+                          -- ^ the channel
+                          -> Processor a b
+                          -- ^ the processor
+queuedChannelProcessor enqueue dequeue f =
+  Processor $ \xs ->
+  Cons $
+  do let composite =
+           do sa <- streamSignal xs
+              sb <- runChannel f sa
+              queuedSignalStream enqueue dequeue sb
+     (ys, h) <- liftEvent $
+                runComposite composite mempty
+     whenCancellingProcess $
+       disposeEvent h
+     runStream ys
+
+-- | Like 'processorChannel' but allows specifying an arbitrary queue for storing the signal values,
+-- for example, the bounded queue.
+queuedProcessorChannel :: (a -> Event ())
+                          -- ^ enqueue
+                          -> (Process a)
+                          -- ^ dequeue
+                          -> Processor a b
+                          -- ^ the processor
+                          -> Channel a b
+                          -- ^ the channel
+queuedProcessorChannel enqueue dequeue (Processor f) =
+  Channel $ \sa ->
+  do xs <- queuedSignalStream enqueue dequeue sa
+     let ys = f xs
+     streamSignal ys
+
 -- | A processor that adds the information about the time points at which 
 -- the original stream items were received by demand.
 arrivalProcessor :: Processor a (Arrival a)
