@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Resource.Preemption
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- This module defines the preemptible resource.
 --
@@ -46,6 +46,7 @@ module Simulation.Aivika.Resource.Preemption
 
 import Data.IORef
 import Data.Monoid
+import Data.Maybe
 
 import Control.Monad
 import Control.Monad.Trans
@@ -128,12 +129,14 @@ newResourceWithMaxCount count maxCount =
   do let r = pointRun p
          t = pointTime p
      when (count < 0) $
-       error $
+       throwIO $
+       SimulationRetry $
        "The resource count cannot be negative: " ++
        "newResourceWithMaxCount."
      case maxCount of
        Just maxCount | count > maxCount ->
-         error $
+         throwIO $
+         SimulationRetry $
          "The resource count cannot be greater than " ++
          "its maximum value: newResourceWithMaxCount."
        _ ->
@@ -308,12 +311,13 @@ releaseResource r =
   Process $ \pid ->
   Cont $ \c ->
   Event $ \p ->
-  do f <- PQ.removeBy (resourceActingQueue r) (\item -> actingItemId item == pid)
+  do f <- fmap isJust $ PQ.queueDeleteBy (resourceActingQueue r) (\item -> actingItemId item == pid)
      if f
        then do invokeEvent p $ updateResourceUtilisationCount r (-1)
                invokeEvent p $ releaseResource' r
                invokeEvent p $ resumeCont c ()
-       else error $
+       else throwIO $
+            SimulationRetry
             "The resource was not acquired by this process: releaseResource"
 
 -- | Release the resource increasing its count and resuming one of the
@@ -327,7 +331,8 @@ releaseResource' r =
      let a' = a + 1
      case resourceMaxCount r of
        Just maxCount | a' > maxCount ->
-         error $
+         throwIO $
+         SimulationRetry $
          "The resource count cannot be greater than " ++
          "its maximum value: releaseResource'."
        _ ->
@@ -348,7 +353,7 @@ releaseResource' r =
                           do PQ.enqueue (resourceActingQueue r) (- priority) $ ResourceActingItem priority pid
                              invokeEvent p $ updateResourceWaitTime r (pointTime p - t)
                              invokeEvent p $ updateResourceUtilisationCount r 1
-                             invokeEvent p $ enqueueEvent (pointTime p) $ resumeCont c ()
+                             invokeEvent p $ enqueueEvent (pointTime p) $ reenterCont c ()
                  Right (ResourcePreemptedItem priority t pid) ->
                    do f <- invokeEvent p $ processCancelled pid
                       case f of
@@ -384,7 +389,8 @@ decResourceCount' r =
   do let t = pointTime p
      a <- readIORef (resourceCountRef r)
      when (a == 0) $
-       error $
+       throwIO $
+       SimulationRetry
        "The resource exceeded and its count is zero: decResourceCount'"
      f <- PQ.queueNull (resourceActingQueue r)
      unless f $
@@ -407,7 +413,7 @@ incResourceCount :: Resource
                     -- ^ the increment for the resource count
                     -> Event ()
 incResourceCount r n
-  | n < 0     = error "The increment cannot be negative: incResourceCount"
+  | n < 0     = throwEvent $ SimulationRetry "The increment cannot be negative: incResourceCount"
   | n == 0    = return ()
   | otherwise =
     do releaseResource' r
@@ -421,7 +427,7 @@ decResourceCount :: Resource
                     -- ^ the decrement for the resource count
                     -> Event ()
 decResourceCount r n
-  | n < 0     = error "The decrement cannot be negative: decResourceCount"
+  | n < 0     = throwEvent $ SimulationRetry "The decrement cannot be negative: decResourceCount"
   | n == 0    = return ()
   | otherwise =
     do decResourceCount' r
