@@ -1,13 +1,13 @@
 
-{-# LANGUAGE CPP, FlexibleContexts, FlexibleInstances, UndecidableInstances, ExistentialQuantification #-}
+{-# LANGUAGE FlexibleContexts, FlexibleInstances, UndecidableInstances, ExistentialQuantification #-}
 
 -- |
 -- Module     : Simulation.Aivika.Results
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The module allows exporting the simulation results from the model.
 --
@@ -29,9 +29,7 @@ module Simulation.Aivika.Results
         ResultComputation(..),
         ResultListWithSubscript(..),
         ResultArrayWithSubscript(..),
-#ifndef __HASTE__
         ResultVectorWithSubscript(..),
-#endif
         -- * Definitions Focused on Using the Library
         ResultValue(..),
         resultsToIntValues,
@@ -107,10 +105,7 @@ import Control.Monad.Trans
 
 import qualified Data.Map as M
 import qualified Data.Array as A
-
-#ifndef __HASTE__
 import qualified Data.Vector as V
-#endif
 
 import Data.Ix
 import Data.Maybe
@@ -134,6 +129,7 @@ import Simulation.Aivika.Server
 import Simulation.Aivika.Activity
 import Simulation.Aivika.Resource
 import qualified Simulation.Aivika.Resource.Preemption as PR
+import Simulation.Aivika.Operation
 import Simulation.Aivika.Results.Locale
 
 -- | A name used for indentifying the results when generating output.
@@ -1822,6 +1818,44 @@ preemptibleResourceResultSummary c =
       resultContainerProperty c "countStats" ResourceCountStatsId PR.resourceCountStats PR.resourceCountChanged_,
       resultContainerProperty c "utilisationCountStats" ResourceUtilisationCountStatsId PR.resourceUtilisationCountStats PR.resourceUtilisationCountChanged_ ] }
 
+-- | Return a source by the specified operation.
+operationResultSource :: ResultContainer (Operation a b)
+                         -- ^ the operation container
+                         -> ResultSource
+operationResultSource c =
+  ResultObjectSource $
+  ResultObject {
+    resultObjectName = resultContainerName c,
+    resultObjectId = resultContainerId c,
+    resultObjectTypeId = OperationId,
+    resultObjectSignal = resultContainerSignal c,
+    resultObjectSummary = operationResultSummary c,
+    resultObjectProperties = [
+      resultContainerProperty c "totalUtilisationTime" OperationTotalUtilisationTimeId operationTotalUtilisationTime operationTotalUtilisationTimeChanged_,
+      resultContainerProperty c "totalPreemptionTime" OperationTotalPreemptionTimeId operationTotalPreemptionTime operationTotalPreemptionTimeChanged_,
+      resultContainerProperty c "utilisationTime" OperationUtilisationTimeId operationUtilisationTime operationUtilisationTimeChanged_,
+      resultContainerProperty c "preemptionTime" OperationPreemptionTimeId operationPreemptionTime operationPreemptionTimeChanged_,
+      resultContainerProperty c "utilisationFactor" OperationUtilisationFactorId operationUtilisationFactor operationUtilisationFactorChanged_,
+      resultContainerProperty c "preemptionFactor" OperationPreemptionFactorId operationPreemptionFactor operationPreemptionFactorChanged_ ] }
+
+-- | Return a summary by the specified operation.
+operationResultSummary :: ResultContainer (Operation a b)
+                          -- ^ the operation container
+                          -> ResultSource
+operationResultSummary c =
+  ResultObjectSource $
+  ResultObject {
+    resultObjectName = resultContainerName c,
+    resultObjectId = resultContainerId c,
+    resultObjectTypeId = OperationId,
+    resultObjectSignal = resultContainerSignal c,
+    resultObjectSummary = operationResultSummary c,
+    resultObjectProperties = [
+      resultContainerProperty c "utilisationTime" OperationUtilisationTimeId operationUtilisationTime operationUtilisationTimeChanged_,
+      resultContainerProperty c "preemptionTime" OperationPreemptionTimeId operationPreemptionTime operationPreemptionTimeChanged_,
+      resultContainerProperty c "utilisationFactor" OperationUtilisationFactorId operationUtilisationFactor operationUtilisationFactorChanged_,
+      resultContainerProperty c "preemptionFactor" OperationPreemptionFactorId operationPreemptionFactor operationPreemptionFactorChanged_ ] }
+
 -- | Return an arbitrary text as a separator source.
 textResultSource :: String -> ResultSource
 textResultSource text =
@@ -1914,16 +1948,12 @@ instance (Show i, Ix i, ResultProvider p) => ResultProvider (A.Array i p) where
       items = A.elems m
       subscript = map (\i -> "[" ++ show i ++ "]") (A.indices m)
 
-#ifndef __HASTE__
-
 instance ResultProvider p => ResultProvider (V.Vector p) where
   
   resultSource' name i m =
     resultSource' name i $ ResultVectorWithSubscript m subscript where
       subscript = V.imap (\i x -> intSubscript i) m
 
-#endif
-
 -- | Represents a list with the specified subscript.
 data ResultListWithSubscript p =
   ResultListWithSubscript [p] [String]
@@ -1932,14 +1962,10 @@ data ResultListWithSubscript p =
 data ResultArrayWithSubscript i p =
   ResultArrayWithSubscript (A.Array i p) (A.Array i String)
 
-#ifndef __HASTE__
-
 -- | Represents a vector with the specified subscript.
 data ResultVectorWithSubscript p =
   ResultVectorWithSubscript (V.Vector p) (V.Vector String)
 
-#endif
-
 instance ResultProvider p => ResultProvider (ResultListWithSubscript p) where
 
   resultSource' name i (ResultListWithSubscript xs ys) =
@@ -1968,8 +1994,6 @@ instance (Show i, Ix i, ResultProvider p) => ResultProvider (ResultArrayWithSubs
     resultSource' name i $ ResultListWithSubscript items subscript where
       items = A.elems xs
       subscript = A.elems ys
-      
-#ifndef __HASTE__
 
 instance ResultProvider p => ResultProvider (ResultVectorWithSubscript p) where
 
@@ -1995,8 +2019,6 @@ instance ResultProvider p => ResultProvider (ResultVectorWithSubscript p) where
         in resultSource' name' (VectorItemId y) x
       items' = V.map resultSourceSummary items
 
-#endif
-
 instance (Ix i, Show i, ResultComputing m) => ResultProvider (m (A.Array i Double)) where
 
   resultSource' name i m =
@@ -2007,8 +2029,6 @@ instance (Ix i, Show i, ResultComputing m) => ResultProvider (m (A.Array i Int))
   resultSource' name i m =
     ResultItemSource $ ResultItem $ fmap A.elems $ computeResultValue name i m
 
-#ifndef __HASTE__
-
 instance ResultComputing m => ResultProvider (m (V.Vector Double)) where
 
   resultSource' name i m =
@@ -2019,8 +2039,6 @@ instance ResultComputing m => ResultProvider (m (V.Vector Int)) where
   resultSource' name i m =
     ResultItemSource $ ResultItem $ fmap V.toList $ computeResultValue name i m
 
-#endif
-
 instance (Show si, Show sm, Show so,
           ResultItemable (ResultValue si),
           ResultItemable (ResultValue sm),
@@ -2062,3 +2080,8 @@ instance ResultProvider PR.Resource where
 
   resultSource' name i m =
     preemptibleResourceResultSource $ ResultContainer name i m (ResultSignal $ PR.resourceChanged_ m)
+
+instance ResultProvider (Operation a b) where
+
+  resultSource' name i m =
+    operationResultSource $ ResultContainer name i m (ResultSignal $ operationChanged_ m)
