@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Results.Locale
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The module defines locales for outputting and printing the simulation results.
 --
@@ -35,6 +35,7 @@ import Simulation.Aivika.Arrival
 import Simulation.Aivika.Server
 import Simulation.Aivika.Activity
 import Simulation.Aivika.Resource
+import Simulation.Aivika.Operation
 
 -- | A locale to output the simulation results.
 --
@@ -234,6 +235,20 @@ data ResultId = TimeId
                 -- ^ Property 'resourceTotalWaitTime'.
               | ResourceWaitTimeId
                 -- ^ Property 'resourceWaitTime'.
+              | OperationId
+                -- ^ Represents an 'Operation'.
+              | OperationTotalUtilisationTimeId
+                -- ^ Property 'operationTotalUtilisationTime'.
+              | OperationTotalPreemptionTimeId
+                -- ^ Property 'operationTotalPreemptionTime'.
+              | OperationUtilisationTimeId
+                -- ^ Property 'operationUtilisationTime'.
+              | OperationPreemptionTimeId
+                -- ^ Property 'operationPreemptionTime'.
+              | OperationUtilisationFactorId
+                -- ^ Property 'operationUtilisationFactor'.
+              | OperationPreemptionFactorId
+                -- ^ Property 'operationPreemptionFactor'.
               | UserDefinedResultId ResultDescription
                 -- ^ An user defined description.
               | LocalisedResultId (M.Map ResultLocale ResultDescription)
@@ -343,6 +358,13 @@ russianResultLocalisation ResourceQueueCountId = "текущая длина оч
 russianResultLocalisation ResourceQueueCountStatsId = "статистика длины очереди к ресурсу"
 russianResultLocalisation ResourceTotalWaitTimeId = "общее время ожидания ресурса"
 russianResultLocalisation ResourceWaitTimeId = "время ожидания ресурса"
+russianResultLocalisation OperationId = "операция"
+russianResultLocalisation OperationTotalUtilisationTimeId = "общее время использования"
+russianResultLocalisation OperationTotalPreemptionTimeId = "общее время вытеснения"
+russianResultLocalisation OperationUtilisationTimeId = "статистика времени использования"
+russianResultLocalisation OperationPreemptionTimeId = "статистика времени вытеснения"
+russianResultLocalisation OperationUtilisationFactorId = "относительное время использования (от 0 до 1)"
+russianResultLocalisation OperationPreemptionFactorId = "относительное время вытеснения (от 0 до 1)"
 russianResultLocalisation (UserDefinedResultId m) = m
 russianResultLocalisation x@(LocalisedResultId m) =
   lookupResultLocalisation russianResultLocale x
@@ -430,9 +452,9 @@ englishResultLocalisation ActivityTotalPreemptionTimeId = "the total time of pre
 englishResultLocalisation ActivityUtilisationTimeId = "the utilisation time"
 englishResultLocalisation ActivityIdleTimeId = "the idle time"
 englishResultLocalisation ActivityPreemptionTimeId = "the preemption time"
-englishResultLocalisation ActivityUtilisationFactorId = "the relative utilisation time (от 0 до 1)"
+englishResultLocalisation ActivityUtilisationFactorId = "the relative utilisation time (from 0 to 1)"
 englishResultLocalisation ActivityIdleFactorId = "the relative idle time (от 0 до 1)"
-englishResultLocalisation ActivityPreemptionFactorId = "the relative preemption time (от 0 до 1)"
+englishResultLocalisation ActivityPreemptionFactorId = "the relative preemption time (from 0 to 1)"
 englishResultLocalisation ResourceId = "the resource"
 englishResultLocalisation ResourceCountId = "the current available count"
 englishResultLocalisation ResourceCountStatsId = "the available count statistics"
@@ -442,6 +464,13 @@ englishResultLocalisation ResourceQueueCountId = "the current queue length"
 englishResultLocalisation ResourceQueueCountStatsId = "the queue length statistics"
 englishResultLocalisation ResourceTotalWaitTimeId = "the total wait time"
 englishResultLocalisation ResourceWaitTimeId = "the wait time"
+englishResultLocalisation OperationId = "the operation"
+englishResultLocalisation OperationTotalUtilisationTimeId = "the total time of utilisation"
+englishResultLocalisation OperationTotalPreemptionTimeId = "the total time of preemption"
+englishResultLocalisation OperationUtilisationTimeId = "the utilisation time"
+englishResultLocalisation OperationPreemptionTimeId = "the preemption time"
+englishResultLocalisation OperationUtilisationFactorId = "the relative utilisation time (from 0 to 1)"
+englishResultLocalisation OperationPreemptionFactorId = "the relative preemption time (from 0 to 1)"
 englishResultLocalisation (UserDefinedResultId m) = m
 englishResultLocalisation x@(LocalisedResultId m) =
   lookupResultLocalisation englishResultLocale x
