@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Simulation
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The module defines the 'Simulation' monad that represents a computation within
 -- the simulation run.
@@ -16,6 +16,7 @@ module Simulation.Aivika.Simulation
         SimulationLift(..),
         runSimulation,
         runSimulations,
+        runSimulationByIndex,
         -- * Error Handling
         catchSimulation,
         finallySimulation,
@@ -24,6 +25,7 @@ module Simulation.Aivika.Simulation
         memoSimulation,
         -- * Exceptions
         SimulationException(..),
-        SimulationAbort(..)) where
+        SimulationAbort(..),
+        SimulationRetry(..)) where
 
 import Simulation.Aivika.Internal.Simulation
