@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Stream
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The infinite stream of data in time.
 --
@@ -23,6 +23,8 @@ module Simulation.Aivika.Stream
         splitStream,
         splitStreamQueueing,
         splitStreamPrioritising,
+        splitStreamFiltering,
+        splitStreamFilteringQueueing,
         -- * Specifying Identifier
         streamUsingId,
         -- * Prefetching and Delaying Stream
@@ -46,6 +48,7 @@ module Simulation.Aivika.Stream
         repeatProcess,
         mapStream,
         mapStreamM,
+        accumStream,
         apStream,
         apStreamM,
         filterStream,
@@ -63,12 +66,18 @@ module Simulation.Aivika.Stream
         -- * Integrating with Signals
         signalStream,
         streamSignal,
+        queuedSignalStream,
         -- * Utilities
         leftStream,
         rightStream,
         replaceLeftStream,
         replaceRightStream,
         partitionEitherStream,
+        -- * Assemblying Streams
+        cloneStream,
+        firstArrivalStream,
+        lastArrivalStream,
+        assembleAccumStream,
         -- * Debugging
         traceStream) where
 
@@ -83,12 +92,13 @@ import Control.Monad.Trans
 import Simulation.Aivika.Simulation
 import Simulation.Aivika.Dynamics
 import Simulation.Aivika.Event
+import Simulation.Aivika.Composite
 import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
 import Simulation.Aivika.Signal
 import Simulation.Aivika.Resource.Base
 import Simulation.Aivika.QueueStrategy
-import Simulation.Aivika.Queue.Infinite
+import qualified Simulation.Aivika.Queue.Infinite.Base as IQ
 import Simulation.Aivika.Internal.Arrival
 
 -- | Represents an infinite stream of data in time,
@@ -213,6 +223,14 @@ mapStreamM f (Cons s) = Cons y where
          b <- f a
          return (b, mapStreamM f xs)
 
+-- | Accumulator that outputs a value determined by the supplied function.
+accumStream :: (acc -> a -> Process (acc, b)) -> acc -> Stream a -> Stream b
+accumStream f acc xs = Cons $ loop xs acc where
+  loop (Cons s) acc =
+    do (a, xs) <- s
+       (acc', b) <- f acc a
+       return (b, Cons $ loop xs acc') 
+
 -- | Sequential application.
 apStream :: Stream (a -> b) -> Stream a -> Stream b
 apStream (Cons sf) (Cons sa) = Cons y where
@@ -343,6 +361,45 @@ splitStreamPrioritising s ps x =
                   return (a, stream ps)
      return $ map stream ps
 
+-- | Split the input stream into the specified number of output streams
+-- after filtering and applying the 'FCFS' strategy for enqueuing the output requests.
+splitStreamFiltering :: [a -> Event Bool] -> Stream a -> Simulation [Stream a]
+splitStreamFiltering = splitStreamFilteringQueueing FCFS
+
+-- | Split the input stream into the specified number of output streams after filtering.
+--
+-- If you don't know what the strategy to apply, then you probably
+-- need the 'FCFS' strategy, or function 'splitStreamFiltering' that
+-- does namely this.
+splitStreamFilteringQueueing :: EnqueueStrategy s
+                                => s
+                                -- ^ the strategy applied for enqueuing the output requests
+                                -> [a -> Event Bool]
+                                -- ^ the filters for output streams
+                                -> Stream a
+                                -- ^ the input stream
+                                -> Simulation [Stream a]
+                                -- ^ the splitted output streams
+splitStreamFilteringQueueing s preds x =
+  do ref <- liftIO $ newIORef x
+     res <- newResource s 1
+     let reader pred =
+           do a <-
+                usingResource res $
+                do p <- liftIO $ readIORef ref
+                   (a, xs) <- runStream p
+                   liftEvent $
+                     do f <- pred a
+                        if f
+                          then do liftIO $ writeIORef ref xs
+                                  return $ Just a
+                          else do liftIO $ writeIORef ref $ Cons (return (a, xs))
+                                  return Nothing
+              case a of
+                Just a  -> return a
+                Nothing -> reader pred
+     return $ map (repeatProcess . reader) preds
+
 -- | Concatenate the input streams applying the 'FCFS' strategy and
 -- producing one output stream.
 concatStreams :: [Stream a] -> Stream a
@@ -496,6 +553,21 @@ prefetchStream s = Cons z where
          spawnProcess $ writer s
          runStream $ repeatProcess reader
 
+-- | Like 'signalStream' but allows specifying an arbitrary queue instead of the unbounded queue.
+queuedSignalStream :: (a -> Event ())
+                      -- ^ enqueue
+                      -> Process a
+                      -- ^ dequeue
+                      -> Signal a
+                      -- ^ the input signal
+                      -> Composite (Stream a)
+                      -- ^ the output stream
+queuedSignalStream enqueue dequeue s =
+  do h <- liftEvent $
+          handleSignal s enqueue
+     disposableComposite h
+     return $ repeatProcess dequeue
+
 -- | Return a stream of values triggered by the specified signal.
 --
 -- Since the time at which the values of the stream are requested for may differ from
@@ -506,32 +578,30 @@ prefetchStream s = Cons z where
 -- The point is that the 'Stream' is requested outside, while the 'Signal' is triggered
 -- inside. They are different by nature. The former is passive, while the latter is active.
 --
--- The resulting stream may be a root of space leak as it uses an internal queue to store
+-- The resulting stream may be a root of space leak as it uses an internal unbounded queue to store
 -- the values received from the signal. The oldest value is dequeued each time we request
--- the stream and it is returned within the computation.
---
--- Cancel the stream's process to unsubscribe from the specified signal.
-signalStream :: Signal a -> Process (Stream a)
+-- the stream and it is returned within the computation. Consider using 'queuedSignalStream' that
+-- allows specifying the bounded queue in case of need.
+signalStream :: Signal a -> Composite (Stream a)
 signalStream s =
-  do q <- liftEvent newFCFSQueue
-     h <- liftEvent $
-          handleSignal s $ 
-          enqueue q
-     whenCancellingProcess $ disposeEvent h
-     return $ repeatProcess $ dequeue q
+  do q <- liftSimulation IQ.newFCFSQueue
+     queuedSignalStream (IQ.enqueue q) (IQ.dequeue q) s
 
--- | Return a computation of the signal that triggers values from the specified stream,
+-- | Return a computation of the disposable signal that triggers values from the specified stream,
 -- each time the next value of the stream is received within the underlying 'Process' 
 -- computation.
---
--- Cancel the returned process to stop reading from the specified stream. 
-streamSignal :: Stream a -> Process (Signal a)
+streamSignal :: Stream a -> Composite (Signal a)
 streamSignal z =
   do s <- liftSimulation newSignalSource
-     spawnProcess $
+     pid <- liftSimulation newProcessId
+     liftEvent $
+       runProcessUsingId pid $
        consumeStream (liftEvent . triggerSignal s) z
+     disposableComposite $
+       DisposableEvent $
+       cancelProcessWithId pid
      return $ publishSignal s
-
+  
 -- | Transform a stream so that the resulting stream returns a sequence of arrivals
 -- saving the information about the time points at which the original stream items 
 -- were received by demand.
@@ -658,6 +728,56 @@ dropStreamWhileM p s =
        then runStream $ dropStreamWhileM p xs
        else return (a, xs)
 
+-- | Create the specified number of equivalent clones of the input stream.
+cloneStream :: Int -> Stream a -> Simulation [Stream a]
+cloneStream n s =
+  do qs  <- forM [1..n] $ \i -> IQ.newFCFSQueue
+     rs  <- newFCFSResource 1
+     ref <- liftIO $ newIORef s
+     let reader m q =
+           do a <- liftEvent $ IQ.tryDequeue q
+              case a of
+                Just a  -> return a
+                Nothing ->
+                  usingResource rs $
+                  do a <- liftEvent $ IQ.tryDequeue q
+                     case a of
+                       Just a  -> return a
+                       Nothing ->
+                         do s <- liftIO $ readIORef ref
+                            (a, xs) <- runStream s
+                            liftIO $ writeIORef ref xs
+                            forM_ (zip [1..] qs) $ \(i, q) ->
+                              unless (i == m) $
+                              liftEvent $ IQ.enqueue q a
+                            return a
+     forM (zip [1..] qs) $ \(i, q) ->
+       return $ repeatProcess $ reader i q
+
+-- | Return a stream of first arrivals after assembling the specified number of elements.
+firstArrivalStream :: Int -> Stream a -> Stream a
+firstArrivalStream n s = assembleAccumStream f (1, Nothing) s
+  where f (i, a0) a =
+          let a0' = Just $ fromMaybe a a0
+          in if i `mod` n == 0
+             then return ((1, Nothing), a0')
+             else return ((i + 1, a0'), Nothing)
+
+-- | Return a stream of last arrivals after assembling the specified number of elements.
+lastArrivalStream :: Int -> Stream a -> Stream a
+lastArrivalStream n s = assembleAccumStream f 1 s
+  where f i a =
+          if i `mod` n == 0
+          then return (1, Just a)
+          else return (i + 1, Nothing)
+
+-- | Assemble an accumulated stream using the supplied function.
+assembleAccumStream :: (acc -> a -> Process (acc, Maybe b)) -> acc -> Stream a -> Stream b
+assembleAccumStream f acc s =
+  mapStream fromJust $
+  filterStream isJust $
+  accumStream f acc s
+
 -- | Show the debug messages with the current simulation time.
 traceStream :: Maybe String
                -- ^ the request message
