@@ -3,11 +3,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Transform
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The module defines something which is most close to the notion of
 -- analogous circuit as an opposite to the digital one.
@@ -84,14 +84,14 @@ instance Arrow Transform where
        c' <- g b
        return $ liftM2 (,) c c'
 
-instance ArrowLoop Transform where
-
-  loop (Transform f) =
-    Transform $ \b ->
-    mdo let bd = liftM2 (,) b d
-        cd <- f bd
-        (c, d) <- unzip0Dynamics cd
-        return c
+-- instance ArrowLoop Transform where
+-- 
+--   loop (Transform f) =
+--     Transform $ \b ->
+--     mdo let bd = liftM2 (,) b d
+--         cd <- f bd
+--         (c, d) <- unzip0Dynamics cd
+--         return c
 
 -- | A transform that returns the current modeling time.
 timeTransform :: Transform a Double
