@@ -1,13 +1,13 @@
 
-{-# LANGUAGE CPP, FlexibleContexts #-}
+{-# LANGUAGE FlexibleContexts #-}
 
 -- |
 -- Module     : Simulation.Aivika.Unboxed
--- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.10.1
+-- Tested with: GHC 8.0.1
 --
 -- The 'Unboxed' class allows creating unboxed arrays in monad 'IO'.
 --
@@ -40,9 +40,5 @@ instance Unboxed Word8
 instance Unboxed Word16
 instance Unboxed Word32
 
-#ifndef __HASTE__
-
 instance Unboxed Int64
 instance Unboxed Word64
-
-#endif
